/* 
 * Copyright (C) 2011-2014 The Regents of the University of California.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * File reading interface that handles gzipped as well as regular files.
 */

#ifndef INFILE_H
#define INFILE_H

//#include <stdio.h>
#ifdef HAVE_LIBZ
# include <zlib.h>
#endif
#ifdef HAVE_PTHREAD
# include <pthread.h>
#endif

class InFile {
    bool isPipe;
    char *tmp;
    FILE *file;
#ifdef HAVE_LIBZ
    gzFile gzfile;
#ifdef HAVE_PTHREAD
    int pipes[2];
    pthread_t pthread;
    static void *run_gzreader(void *arg);
#endif
    void check_gzerror();
#endif
public:
    static bool fork;
    const char * const name;
    const char * basename;
    explicit InFile(const char *filename);
    ~InFile() { close(); }
    char *gets(char *buf, unsigned len) throw();
    size_t read(void *buf, size_t size, size_t nmemb) throw();
    int fd() throw();
    void close();
    class Error {
	char *buf;
      public:
	Error(const char *fmt, ...) throw();
	~Error() throw() {
	    delete[] buf;
	}
	const char *what() { return buf; }
    };
    bool nameEndsWith(const char *ending);
};

#endif // INFILE_H
